#!/usr/bin/python
# -*- coding: utf-8 -*-

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
# Author: Tomas "Tojaj" Mlcoch (xtojaj gmail dot com)
# Web: http://tojaj.com
#
# Copyright 2010 Tomas Mlcoch


"""Module for movie identification
via opensubtitles.org RPC. 
Using three methods of identification.
In order:
1) Movie hash calculated from movie file.
2) IMDb ID from *.nfo file if present.
3) Assess movie name from name of movie file.
"""

import sys
import os
import re
import fnmatch

import rpcclient
import progress
from core import hashfile


# RegExp for something like this: "http://www.imdb.com/title/tt0208092/"
RE_IMDBID = re.compile(r'.*imdb\.[^\/]+\/title\/tt(?P<id>\d+).*', re.IGNORECASE)

# RexExp for year
RE_YEAR = re.compile(r'(19|20)\d\d')


def identify(movies, verbose=False):
    """ Identify movies in list 
    Expected format of list items:
    {'Path':'/foo/bar','Filename':'foofile.ext' }
    """
    rpcobj = rpcclient.OsRpcClient()

    indicator = progress.ProgressLine(len(movies), 'Identification: ')
    for movie in movies:
        verbose and indicator.next()
        
        filepath = movie['Path'] + "/" + movie['Filename']

        # Filehash based identification
        try:
            msize = os.path.getsize(filepath)
            mhash = hashfile.hashFile(filepath)
        except (IOError):
            continue

        movie.update({'FileSize':msize})

        movie_ident = None
        movie_ident = rpcobj.movie_by_hash(mhash)
        if movie_ident:
            movie.update(movie_ident)
            continue

        # *.nfo file based identification
        nfo_file = None
        for mfile in os.listdir(movie['Path']):
            if fnmatch.fnmatch(mfile, '*.nfo'):
                if nfo_file:
                    # More than one *.nfo file in one folder => skip
                    nfo_file = None
                    break
                nfo_file = movie['Path'] + '/' + mfile

        if nfo_file:
            imdbid = None
            try:
                for line in open(nfo_file):
                    if RE_IMDBID.match(line): 
                        imdbid = RE_IMDBID.match(line).group('id')
                        break
            except(IOError):
                pass
            
            if imdbid:
                movie_ident = rpcobj.movie_by_imdbid(imdbid)
                if movie_ident:
                    movie.update(movie_ident)
                    continue

        # Filename based indentification
        reg = re.compile(r'\W+', re.UNICODE)
        words = reg.split(unicode(movie['Filename'], 'utf-8'))[:-1]
        if words:
            movie_name = ''
            for word in words:
                word = word.encode('utf-8')
                if RE_YEAR.match(word):
                    break
                movie_name += word + ' ' 
            movie_name = movie_name.strip()
            if not movie_name:
                movie_name = words[0]

            movie_ident = rpcobj.movie_by_name(movie_name)
            if movie_ident:
                movie.update(movie_ident)
                continue
            movie.update({'GuessedName':movie_name})


if __name__ == '__main__':
    # pylint: disable-msg=C0103
    import sys
    if len(sys.argv) == 1:
        sys.exit(0)

    parts = sys.argv[1].rpartition('/')
    film = {}
    film['Path'] = parts[0]+parts[1] or os.getcwd()
    film['Filename'] = sys.argv[1].rsplit('/', 1)[-1] 

    films = []
    films.append(film)
    identify(films)
    for key, data in films[0].iteritems():
        print "{0}: {1}".format(key, data)
